/**********************************************
Macros lorenz gini.sas

Macros SAS de reprsentation de la courbe de 
Lorenz et de calcul de l'indice de Gini

CEPE - Certificat Charg d'tudes statistiques
Martin Chevalier
28/11/2016
**********************************************/


%MACRO lorenz(DATA =, VAR =);

%LOCAL n sum;

/*Etape 1 : Tri selon &VAR*/
PROC SORT DATA = &DATA(KEEP = &VAR WHERE = (&VAR NE .)) OUT = _lorenz_; 
	BY &VAR;
RUN;

/*Etape 2 : Calcul de statistiques*/
PROC SQL NOPRINT;
	SELECT COUNT(*), SUM(&VAR) INTO :n, :sum FROM _lorenz_;
QUIT;

/*Etape 3 : Calcul des coordonnes*/
DATA _lorenz_;
	SET _lorenz_;
	x = _N_ / &n;
	RETAIN y 0;
	y = y + &VAR / &sum;
RUN;

/*Etape 4 : Reprsentation graphique*/
AXIS1 OFFSET = (0);
AXIS2 OFFSET = (0) LABEL = (ANGLE = 90);
SYMBOL1 I = JOIN;
SYMBOL2 I = JOIN;
PROC GPLOT DATA = _lorenz_;
	PLOT y * x = 1 x * x = 2 / OVERLAY HAXIS = AXIS1 VAXIS = AXIS2;
RUN; QUIT;

/*Etape 5 : Nettoyage*/
PROC SQL NOPRINT; 
	DROP TABLE _lorenz_;
QUIT;
%MEND;


%MACRO gini(DATA =, VAR =);

%LOCAL n sum gini;

/*Etape 1 : Tri selon &VAR*/
PROC SORT DATA = &DATA(KEEP = &VAR WHERE = (&VAR NE .)) OUT = _gini_; 
	BY &VAR;
RUN;

/*Etape 2 : Calcul de statistiques*/
PROC SQL NOPRINT;
	SELECT COUNT(*), SUM(&VAR) INTO :n, :sum FROM _gini_;
QUIT;

/*Etape 3 : Calcul de l'aire sous la courbe de Lorenz*/
DATA _gini_;
	SET _gini_;
	RETAIN y 0;
	y = y + &VAR / &sum;
	lag_y = MAX(0,LAG(y)); 
	aire = ( 1 / &n ) * ( lag_y + (y - lag_y) / 2 );
RUN;
PROC SQL NOPRINT;
	SELECT 1 - 2 * SUM(aire) INTO :gini FROM _gini_;
QUIT;
%PUT Indice de Gini associ  la variable &VAR de la base &DATA %STR(:) &gini..;

/*Etape 4 : Nettoyage*/
PROC SQL NOPRINT; 
	DROP TABLE _gini_;
QUIT;

%MEND;
